function get_acc, t, x, y, z, height, lat, lon, vx, vy, vz, aeroaccsctx, aeroaccscty, aeroaccsctz, omega

; Paul Withers, 2001.07.23
; Open University, Britain

; Called by:				recon_traj.pro, derivs.pro
; Calls:				get_grav
; Declares common blocks:		N/A
; Reads from:				N/A
; Writes to:				N/A
; Options:				N/A
; Purpose:				Calculate accelerations in spacecraft 
;					frame using many pieces of 
;					information. 

common sctblk

forward_function get_grav

trustxy = sctpropsstruct.trustxy

vwind = crossp(double([0.,0.,1.])*omega, [x,y,z])
; Calculate wind velocity in inertial cartesian frame at current position

zonalwind=(0.) ; which way is "zonal"???
vwind = vwind + [(-1.)*zonalwind*sin(atan(y,x)), zonalwind*cos(atan(y,x)), 0.]

vxrel = vx - vwind(0)
vyrel = vy - vwind(1)
vzrel = vz - vwind(2)
; Calculate spacecraft velocity relative to atmosphere in 
; inertial cartesian frame
aeroacc = sqrt( trustxy*(aeroaccsctx^2 + aeroaccscty^2) + aeroaccsctz^2 )
vrel = sqrt( vxrel^2 + vyrel^2 + vzrel^2 )
; Calculate magnitudes of aerodynamic acceleration and relative velocity
aeroaccinex = double(-1.) * aeroacc * vxrel / vrel
aeroacciney = double(-1.) * aeroacc * vyrel / vrel
aeroaccinez = double(-1.) * aeroacc * vzrel / vrel
; Calculate xyz components of aerodynamic acceleration in inertial cartesian 
; frame
; Assume that aerodynamic forces are opposed to relative velocity vector
; and so express them in inertial frame
; Also assume that all aerodynamic forces act along sct-z axis
; aka ballistic trajectory, lift-free trajectory
; This has a bunch of assumptions about angle-of-attack, drag coeff, and so on

g = get_grav(height, lat, lon, omega*t)
; Calculate acceleration due to gravity at
; spacecraft position in inertial cartesian frame

return, [vx, vy, vz, $
 (aeroaccinex+g(0)), (aeroacciney+g(1)),(aeroaccinez+g(2)), $
 vwind(0), vwind(1), vwind(2), $
 vxrel, vyrel, vzrel, aeroacc, vrel, $
 aeroaccinex, aeroacciney, aeroaccinez, $
 g(0), g(1), g(2)]
; First six elements are necessary for trajectory integration, 
; rest are housekeeping
end
