function get_cd, kn, ma, v, aratio, i

; Paul Withers, 2005.05.11
; Center for Space Physics, Boston University

; Generate aerodynamic database from AIAA-2005-0056
; plus Schoenenberger email...

common sctblk
; Properties of spacecraft and datasets
; Defined in sctprops

alpha_fmf = sctpropsstruct.aerodb.alpha_fmf
na_fmf = sctpropsstruct.aerodb.na_fmf
ca_fmf = sctpropsstruct.aerodb.ca_fmf
cn_fmf = sctpropsstruct.aerodb.cn_fmf
cm_fmf = sctpropsstruct.aerodb.cm_fmf
alpha_trans = sctpropsstruct.aerodb.alpha_trans
na_trans = sctpropsstruct.aerodb.na_trans
kn_trans = sctpropsstruct.aerodb.kn_trans
nkn_trans = sctpropsstruct.aerodb.nkn_trans
ca_transalphakn = sctpropsstruct.aerodb.ca_transalphakn
cn_transalphakn = sctpropsstruct.aerodb.cn_transalphakn
cm_transalphakn = sctpropsstruct.aerodb.cm_transalphakn
alpha_cont = sctpropsstruct.aerodb.alpha_cont
na_cont = sctpropsstruct.aerodb.na_cont
v_cont = sctpropsstruct.aerodb.v_cont
nv_cont = sctpropsstruct.aerodb.nv_cont
ma_cont = sctpropsstruct.aerodb.ma_cont
nma_cont = sctpropsstruct.aerodb.nma_cont
ca_cont = sctpropsstruct.aerodb.ca_cont
cn_cont = sctpropsstruct.aerodb.cn_cont
cm_cont = sctpropsstruct.aerodb.cm_cont
knfmf = sctpropsstruct.aerodb.knfmf
kntoptrans = sctpropsstruct.aerodb.kntoptrans
knbottrans = sctpropsstruct.aerodb.knbottrans
kncont = sctpropsstruct.aerodb.kncont
matopweight = sctpropsstruct.aerodb.matopweight
mabotweight = sctpropsstruct.aerodb.mabotweight
alphaerr = sctpropsstruct.aerodb.alphaerr
uncert = sctpropsstruct.uncert

; ***** NEED TO REGRID SOMETHING FOR USE AT BOUNDARY *****
; ***** OF CONT AND TRANS REGIONS *****
ca_endtrans = interpol(ca_transalphakn(*,0), alpha_trans, alpha_cont)
cn_endtrans = interpol(cn_transalphakn(*,0), alpha_trans, alpha_cont)
cm_endtrans = interpol(cm_transalphakn(*,0), alpha_trans, alpha_cont)

; outputs
cnoverca = 0.
ca = 0.
cn = 0.
cm = 0.
alpha = 0.

flag=0.



; ***** WANDER IN MA/V/KN FOREST *****

; Since I can deal with getting first element in array, but not last element in array
; Use ge/lt choices...

; ***** FMF *****
if kn ge knfmf then begin
cnovercamix = cn_fmf / ca_fmf
camix = ca_fmf
cnmix = cn_fmf
cmmix = cm_fmf
alphamix = alpha_fmf
flag=1
endif ; if kn ge knfmf

; ***** TRANS/FMF *****
if kn ge kntoptrans and kn lt knfmf then begin
; gt 1e2 fmf to toptrans
; 1e-1 to 1e2 within trans
; 1.06e-3 to 1e-1 topcont to bottom trans
knhi = knfmf
cahi = ca_fmf
cnhi = cn_fmf
cmhi = cm_fmf
knlo = kntoptrans
calo = ca_transalphakn(*,nkn_trans-1)
cnlo = cn_transalphakn(*,nkn_trans-1)
cmlo = cm_transalphakn(*,nkn_trans-1)

gg = (alog(kn) - alog(knlo)) / (alog(knhi) - alog(knlo))
cnovercamix = gg * cnhi / cahi + (1.-gg) * cnlo / calo
camix = gg * cahi + (1.-gg) * calo
cnmix = gg * cnhi + (1.-gg) * cnlo
cmmix = gg * cmhi + (1.-gg) * cmlo

alphamix = alpha_fmf ; same for both fmf and trans
; should do this a bit better...
flag=1
endif ; if kn ge kntoptrans and kn lt knfmf

; ***** TRANS *****
if kn ge knbottrans and kn lt kntoptrans then begin
hh = where(kn ge kn_trans)
knlo = kn_trans(max(hh))
calo = ca_transalphakn(*,max(hh))
cnlo = cn_transalphakn(*,max(hh))
cmlo = cm_transalphakn(*,max(hh))
knhi = kn_trans(max(hh)+1) 
; if I use gt/le, then I run risk of max(hh) being last element and +1 blowing it up
cahi = ca_transalphakn(*,max(hh)+1)
cnhi = cn_transalphakn(*,max(hh)+1)
cmhi = cm_transalphakn(*,max(hh)+1)

gg = (alog(kn) - alog(knlo)) / (alog(knhi) - alog(knlo))
cnovercamix = gg * cnhi / cahi + (1.-gg) * cnlo / calo
camix = gg * cahi + (1.-gg) * calo
cnmix = gg * cnhi + (1.-gg) * cnlo
cmmix = gg * cmhi + (1.-gg) * cmlo

alphamix = alpha_fmf ; same for both fmf and trans
flag=1
endif ; if kn ge knbottrans and kn lt kntoptrans

; ***** CONT/TRANS *****
if kn ge kncont and kn lt knbottrans then begin
knhi = knbottrans
cahi = ca_endtrans
cnhi = cn_endtrans
cmhi = cm_endtrans
knlo = kncont
calo = ca_cont(*,nv_cont-1)
cnlo = cn_cont(*,nv_cont-1)
cmlo = cm_cont(*,nv_cont-1)

gg = (alog(kn) - alog(knlo)) / (alog(knhi) - alog(knlo))
cnovercamix = gg * cnhi / cahi + (1.-gg) * cnlo / calo
camix = gg * cahi + (1.-gg) * calo
cnmix = gg * cnhi + (1.-gg) * cnlo
cmmix = gg * cmhi + (1.-gg) * cmlo

alphamix = alpha_cont 
flag=1
endif ; if kn ge kncont and kn lt knbottrans

; ***** CONT *****
if kn lt kncont then begin
; Into the continuum mess...

; ***** MA > MAX(MA) *****
if ma ge max(ma_cont) then begin
; once ma says in this regime, take values from edge simulation
cnovercamix = cn_cont(*,nv_cont-1) / ca_cont(*,nv_cont-1)
camix = ca_cont(*,nv_cont-1)
cnmix = cn_cont(*,nv_cont-1)
cmmix = cm_cont(*,nv_cont-1)
alphamix = alpha_cont
flag=1
endif ; if v ge max(v_cont)

; ***** MA > HI-MA END OF WEIGHTING ENVELOPE AND MA < MAX(MA) *****
if ma ge matopweight and ma lt max(ma_cont) then begin
hh = where(v ge v_cont)
vlo = v_cont(max(hh))
calo = ca_cont(*,max(hh))
cnlo = cn_cont(*,max(hh))
cmlo = cm_cont(*,max(hh))
vhi = v_cont(max(hh)+1) 
cahi = ca_cont(*,max(hh)+1)
cnhi = cn_cont(*,max(hh)+1)
cmhi = cm_cont(*,max(hh)+1)

gg = (alog(v) - alog(vlo)) / (alog(vhi) - alog(vlo))
cnovercamix = gg * cnhi / cahi + (1.-gg) * cnlo / calo
camix = gg * cahi + (1.-gg) * calo
cnmix = gg * cnhi + (1.-gg) * cnlo
cmmix = gg * cmhi + (1.-gg) * cmlo

alphamix = alpha_cont ; same for both fmf and trans
flag=1
endif ; if ma ge matopweight 


; ***** MA > LO-MA END OF WEIGHTING ENVELOPE AND ***** 
; ***** MA < HI-MA END OF WEIGHTING ENVELOPE  *****
if ma ge mabotweight and ma lt matopweight then begin
; Messy mixed v/Ma averaging...
hhv = where(v ge v_cont)
vlo = v_cont(max(hhv))
calov = ca_cont(*,max(hhv))
cnlov = cn_cont(*,max(hhv))
cmlov = cm_cont(*,max(hhv))
vhi = v_cont(max(hhv)+1) 
cahiv = ca_cont(*,max(hhv)+1)
cnhiv = cn_cont(*,max(hhv)+1)
cmhiv = cm_cont(*,max(hhv)+1)

hhma = where(ma ge ma_cont)
malo = ma_cont(max(hhma))
caloma = ca_cont(*,max(hhma))
cnloma = cn_cont(*,max(hhma))
cmloma = cm_cont(*,max(hhma))
mahi = ma_cont(max(hhma)+1) 
cahima = ca_cont(*,max(hhma)+1)
cnhima = cn_cont(*,max(hhma)+1)
cmhima = cm_cont(*,max(hhma)+1)

ggv = (alog(v) - alog(vlo)) / (alog(vhi) - alog(vlo))
ggma = (alog(ma) - alog(malo)) / (alog(mahi) - alog(malo))

ii = (ma - mabotweight) / (matopweight - mabotweight)
; ma used in switch from v/ma control
cnovercamix = ii * (ggv*cnhiv/cahiv + (1.-ggv)*cnlov/calov) + $
 (1.-ii) * (ggma*cnhima/cahima + (1.-ggma)*cnloma/caloma)
camix = ii * (ggv*cahiv + (1.-ggv)*calov) + $
 (1.-ii) * (ggma*cahima + (1.-ggma)*caloma)
cnmix = ii * (ggv*cnhiv + (1.-ggv)*cnlov) + $
 (1.-ii) * (ggma*cnhima + (1.-ggma)*cnloma)
cmmix = ii * (ggv*cmhiv + (1.-ggv)*cmlov) + $
 (1.-ii) * (ggma*cmhima + (1.-ggma)*cmloma)
alphamix = alpha_cont
flag=1
endif ; if ma ge mabotweight and ma lt matopweight

; ***** MA > MIN(MA) AND MA < LO-MA END OF WEIGHTING ENVELOPE *****
if ma ge min(ma_cont) and ma lt mabotweight then begin
hh = where(ma ge ma_cont)
malo = ma_cont(max(hh))
calo = ca_cont(*,max(hh))
cnlo = cn_cont(*,max(hh))
cmlo = cm_cont(*,max(hh))
mahi = ma_cont(max(hh)+1) 
cahi = ca_cont(*,max(hh)+1)
cnhi = cn_cont(*,max(hh)+1)
cmhi = cm_cont(*,max(hh)+1)

gg = (alog(ma) - alog(malo)) / (alog(mahi) - alog(malo))
cnovercamix = gg * cnhi / cahi + (1.-gg) * cnlo / calo
camix = gg * cahi + (1.-gg) * calo
cnmix = gg * cnhi + (1.-gg) * cnlo
cmmix = gg * cmhi + (1.-gg) * cmlo

alphamix = alpha_cont ; same for both fmf and trans
flag=1
endif ; if ma ge min(ma_cont) and ma lt mabotweight

; ***** MA < MIN(MA) *****
if ma lt min(ma_cont) then begin
cnovercamix = cn_cont(*,0) / ca_cont(*,0)
camix = ca_cont(*,0)
cnmix = cn_cont(*,0)
cmmix = cm_cont(*,0)

alphamix = alpha_cont ; same for both fmf and trans
flag=1
endif ; if ma lt min(ma_cont)

endif ; if kn lt kncont 

; ***** HAVE FOUND LOCATION IN MA/V/KN FOREST *****
if flag eq 0 then stop ; problem...


; ***** ALPHA < MIN(ALPHA) *****
; Having formed relevant average, use it.
if aratio lt min(cnovercamix) then begin
cnoverca = cnovercamix(0)
ca = camix(0)
cn = cnmix(0)
cm = cmmix(0)
alpha = alphamix(0)
endif 

; ***** ALPHA > MIN(ALPHA) AND ALPHA < MAX(ALPHA) *****
if aratio ge min(cnovercamix) and aratio lt max(cnovercamix) then begin
aa = where(aratio ge cnovercamix)
; max(aa) to max(aa)+1
ff = (aratio - cnovercamix(max(aa))) / (cnovercamix(max(aa)+1) - cnovercamix(max(aa)))
cnoverca = ff * cnovercamix(max(aa)+1) + (1.-ff) * cnovercamix(max(aa)) 
ca = ff * camix(max(aa)+1) + (1.-ff) * camix(max(aa)) 
cn = ff * cnmix(max(aa)+1) + (1.-ff) * cnmix(max(aa)) 
cm = ff * cmmix(max(aa)+1) + (1.-ff) * cmmix(max(aa)) 
alpha = ff * alphamix(max(aa)+1) + (1.-ff) * alphamix(max(aa)) 
endif

; ***** ALPHA > MAX(ALPHA) *****
if aratio ge max(cnovercamix) then begin
cnoverca = cnovercamix(n_elements(alphamix)-1)
ca = camix(n_elements(alphamix)-1)
cn = cnmix(n_elements(alphamix)-1)
cm = cmmix(n_elements(alphamix)-1)
alpha = alphaerr ; so I know it's hiccupped
;alphamix(n_elements(alphamix)-1)
endif

;ca = cn/cnoverca
; Doesn't seem to need it, CN is OK.

; Find a way to fix alpha
blah=0
if blah ne 0 then begin
targetalpha = 5.
aa = where(alphamix eq targetalpha)
if aa(0) ne -1 then begin
cnoverca = cnovercamix(aa(0))
ca = camix(aa(0))
cn = cnmix(aa(0))
cm = cmmix(aa(0))
alpha = alphamix(aa(0))
endif else begin
aa = where(alphamix lt targetalpha)
ff = (targetalpha - alphamix(max(aa))) / (alphamix(max(aa+1))-alphamix(max(aa)))
cnoverca = ff * cnovercamix(max(aa)+1) + (1.-ff) * cnovercamix(max(aa)) 
ca = ff * camix(max(aa)+1) + (1.-ff) * camix(max(aa)) 
cn = ff * cnmix(max(aa)+1) + (1.-ff) * cnmix(max(aa)) 
cm = ff * cmmix(max(aa)+1) + (1.-ff) * cmmix(max(aa)) 
alpha = ff * alphamix(max(aa)+1) + (1.-ff) * alphamix(max(aa)) 
endelse
endif

return, [ca, cn, alpha]
; no use for cm, cnoverca at present

end
