function get_entry_state

; Paul Withers, 2001.07.12
; Open University, Britain

; Called by:				recon_traj.pro, model_atm.pro
; Calls:				get_omega, get_uncert
; Declares common blocks:		N/A
; Reads from:				N/A
; Writes to:				N/A
; Options:				initial_conditions = 'jsr','pds','b2'
; Purpose:				Convert initial conditions into 
;					inertial cartesian frame

common marsblk
; Properties of Mars and physical constants
; Defined in marsprops

common sctblk
; Properties of spacecraft and datasets
; Defined in sctprops

t = sctpropsstruct.entrystate.t
r = sctpropsstruct.entrystate.r
lat = sctpropsstruct.entrystate.lat
lon = sctpropsstruct.entrystate.lon
speed = sctpropsstruct.entrystate.speed
gamma = sctpropsstruct.entrystate.gamma
psi = sctpropsstruct.entrystate.psi
rot_factor = sctpropsstruct.entrystate.rot_factor
st = sctpropsstruct.entrystate.st
sr = sctpropsstruct.entrystate.sr
slat = sctpropsstruct.entrystate.slat
slon = sctpropsstruct.entrystate.slon
sspeed = sctpropsstruct.entrystate.sspeed
sgamma = sctpropsstruct.entrystate.sgamma
spsi = sctpropsstruct.entrystate.spsi

omega = marspropsstruct.omega ; Look up planetary rotation rate

uncert = sctpropsstruct.uncert

print, 'Entry state is '
print, '(s) t = ', t, ' +/- ', st
print, '(m) r = ', r, ' +/- ', sr
print, '(degN) lat = ', lat, ' +/- ', slat
print, '(degE) lon = ', lon, ' +/- ', slon
print, '(m s-1) speed = ', speed, ' +/- ', sspeed
print, '(deg below horizontal) gamma = ', gamma, ' +/- ', sgamma
print, '(deg clockwise from north) psi = ', psi, ' +/- ', spsi
print, 'Frame has rot_factor of ', rot_factor

; testing shift trajectory idea
; subtract my nominal landed site, add JPL landed site
;r = r - double(3389213.0) + double(3389715.)
;lat = lat - double(19.054172) + double(19.09)
;lon = lon - double(326.445) + double(326.48)

t 	= t 	+ st*randomn(seed)*uncert
r 	= r 	+ sr*randomn(seed)*uncert
lat 	= lat 	+ slat*randomn(seed)*uncert
lon 	= lon 	+ slon*randomn(seed)*uncert
speed	= speed + sspeed*randomn(seed)*uncert
gamma	= gamma + sgamma*randomn(seed)*uncert
psi 	= psi 	+ spsi*randomn(seed)*uncert

colat = (double(90.)-lat)*double(!pi/180.)
lon   = lon  *double(!pi/180.)
gamma = gamma*double(!pi/180.)
psi   = psi  *double(!pi/180.)
; Convert angles from degrees to radians

v_r = double(-1.) * speed*sin(gamma)
v_colat = double(-1.) * speed*cos(gamma)*cos(psi)
v_lon = speed*cos(gamma)*sin(psi)
; Transform velocity into momentary spherical frame

; Now need to subtract rotational component of speed.
v_lon=v_lon + omega*r*sin(colat)*rot_factor

v_intx = v_r*sin(colat)*cos(lon) + v_colat*cos(colat)*cos(lon) - v_lon*sin(lon)
v_inty = v_r*sin(colat)*sin(lon) + v_colat*cos(colat)*sin(lon) + v_lon*cos(lon)
v_intz = v_r*cos(colat)          - v_colat*sin(colat)
; This transformation from Bradbury, Theoretical Mechanics, p64
; It's a standard one
; Momentary spherical frame to momentary cartesian frame

angle = omega*t
vx1 = v_intx * cos(angle) - v_inty * sin(angle)
vy1 = v_intx * sin(angle) + v_inty * cos(angle)
vz1 = v_intz
; Transform from momentary cartesian frame to inertian cartesian frame

lon = lon + angle
; Transfrom longitude from momentary spherical frame to 
; inertial spherical frame

z1 = r*cos(colat)
y1 = r*sin(colat)*sin(lon)
x1 = r*sin(colat)*cos(lon)
; Transform position from inertial spherical frame into 
; inertial cartesian frame

return, [x1, y1, z1, vx1, vy1, vz1, t]
; Return position, velocity, and time in inertial cartesian frame
end
