pro marsprops

; Paul Withers, 2005.05.13
; Center for Space Physics, Boston University

; Make common block of Mars properties

common marsblk, marspropsstruct
; common block called marsblk
; contents are one structure, marspropsstruct

marspropsstruct = {marspropsstruct, gm:double(0.0), c20:double(0.0), rref:double(0.0), omega:double(0.0), mean_molecular_mass:double(0.0), universal_gas_constant:double(0.0), avogadro:double(0.0), moldiameter:double(0.0), ratioheats:double(0.0)}

gm = double(42828.382332 * 1e9)   
marspropsstruct.gm = gm
print, '(m3 s-2) GM = ', gm     
; Grav constant*Mass of Mars in m^3/s^2, PDS dataset mors_1006

rref = double(3394.2 * 1e3)
marspropsstruct.rref = rref
print, '(m) rref = ', rref     
; Reference radius of Mars in m, PDS dataset mors_1006

c20 = double(-1. * 8.75981 * 1e-4) 
c20 = c20 *sqrt(5.)
marspropsstruct.c20 = c20
print, '(-) c20 = ', c20     
; C20 dimensionless coefficient
; Normalization conventions are hideous, see Kaula book (1966)
; and Smith et al to understand. Also some Tolson/Withers emails.

omega = double(2.*!pi / (24.6229 *60.*60.))
marspropsstruct.omega = omega
print, '(rad s-1) omega = ', omega     
; http://nssdc.gsfc.nasa.gov/planetary/factsheet/marsfact.html
; Sidereal rot rate in rad s-1

mean_molecular_mass = double(43.49*1e-3)
marspropsstruct.mean_molecular_mass = mean_molecular_mass
print, '(kg mol-1) mean_molecular_mass = ', mean_molecular_mass
; Mean molecular mass (assumed uniform in atmosphere) in kg mol-1
; Magalhaes et al, 1999

universal_gas_constant = double(8.31451)
marspropsstruct.universal_gas_constant = universal_gas_constant
print, '(J K-1 mol-1) universal_gas_constant = ', universal_gas_constant
; Universal gas constant in J K-1 mol-1, Lodders and Fegley

avogadro = double(6.02e23) ; particles per mole
marspropsstruct.avogadro = avogadro
print, '(mol-1) Avogadros number = ', avogadro
; Avogadro's number, Lodders and Fegley

moldiameter = double(4.64e-10) ; Molecular diameter (CO2) in m
marspropsstruct.moldiameter = moldiameter
print, '(m) Molecular diameter = ', moldiameter
; Schoenenbeger et al. AIAA-2005-0056

ratioheats = double(7./5.) ; Ratio of specific heats for CO2
marspropsstruct.ratioheats = ratioheats
print, '(-) Ratio of specific heats = ', ratioheats
; Ideal linear polyatomic gas, Atkins Physical Chemistry
; and orca.phys.uvic.ca/~tatum/phys317/notes/8P317heatcap.pdf 

help, marspropsstruct, /struct

end
