pro sctprops_descoped

; Paul Withers, 2005.05.13
; Center for Space Physics, Boston University

; Spacecraft-specific properties
; Make common block of spacecraft-specific properties

common sctblk, sctpropsstruct, aerosigmatracker
; common block called sctblk
; contents are one structure, sctpropsstruct
; and another one, aerosigmatracker

sctpropsstruct = {sct:'', pdsfile:'', datafile:'', outdir:'', dt:double(0.0), nraw:0L, n:0L, tchute:double(0.0), tdetect:double(0.0), nchute:0L, ndetect:0L, npadtop:0L, nsection:double(0.0), nsplit:0L, i0:0L, i1:0L, trustxy:double(0.0), sa:double(0.0), alimit:double(0.0), m:double(0.0), a:double(0.0), r0:double(0.0), range_0:double(0.0), height_0:double(0.0), convergence:double(0.0), uncert:double(0.0), nmonte_traj:double(0.0), nmonte_atm:double(0.0), nomt0:double(0.0), snomt0:double(0.0), nalpha:0L, nblocks:0L, blocksize:0L, nmonte_hiz:double(0.0), zerofraclimit:double(0.0), badrunlimit:double(0.0), t0string:'', itlimit:double(0.0)}
; add entrystate and aerodb at end...

sct='mer2'
sctpropsstruct.sct = sct
print, '(-) Sct = ', sct
; Name of spacecraft

uncert = double(0.0)
sctpropsstruct.uncert = uncert
print, '(-) uncert = ', uncert
; 0 means do not consider uncertainties
; 1 means consider uncertainties 


if sct eq 'mer1' then begin
pdsfile = '1TRANSFORMED.TAB'
sctpropsstruct.pdsfile = pdsfile
print, '(-) pdsfile = ', pdsfile
; Location of raw data
	
nraw = 9050L
sctpropsstruct.nraw = nraw
print, '(-) nraw = ', nraw
; Number of lines in raw datafile

nsplit = 5218L
sctpropsstruct.nsplit = nsplit
print, '(-) nsplit = ', nsplit
; Backshell separates and IMU-B data ends

;tdetect = 8219.6250
;sctpropsstruct.tdetect = tdetect
;print, '(s) tdetect = ', tdetect
; Time of atm detection, determined manually

;tchute = 8444.875
;sctpropsstruct.tchute = tchute
;print, '(s) tchute = ', tchute
; Time of chute opening determined manually

r0 = double(3394.1e3) ; MER-1 from MOLA/NSSDC
sctpropsstruct.r0 = r0
print, '(m) Planetary radius at impact site = ', r0

if uncert eq 1 then height_0 = r0 + 100e3
if uncert eq 0 then height_0 = r0 + 120e3
sctpropsstruct.height_0 = height_0
print, '(m) height_0 = ', height_0
; Top of region used for Hfit in top BC
; The atmospheric structure reconstruction is very sensitive to this

npadtop = 80L
sctpropsstruct.npadtop = npadtop
print, '(-) npadtop = ', npadtop
; Difference between top of uncert=0 and uncert=1 atmo recons

ndetect = 20L
if uncert eq 1 then ndetect = ndetect + npadtop
sctpropsstruct.ndetect = ndetect
print, '(-) ndetect = ', ndetect
; atm detection, determined manually

nchute = 1001L
sctpropsstruct.nchute = nchute
print, '(-) nchute = ', nchute
; chute opening determined manually
; *** Use n, not t, for these two, else problems in recon_atm ***

nalpha = 190L 
sctpropsstruct.nalpha = nalpha
print, '(-) nalpha = ', nalpha
; Where do I start considering alpha?

i0 = 4674
sctpropsstruct.i0 = i0
print, '(-) i0 = ', i0
i1 = 4675
sctpropsstruct.i1 = i1
print, '(-) i1 = ', i1
; Couple of blank datapoints that need filling in by read_pds.pro

m = double(832.2)
sctpropsstruct.m = m
print, '(kg) Sct mass = ', m
; Mass of spacecraft, kg
; Desai AIAA-2004-5092, fig2

nomt0 = double(160.)
sctpropsstruct.nomt0 = nomt0
print, '(K) nomt0 = ', nomt0
; DSH fitted temp at height_0 using nominal recon, no uncert.

snomt0 = double(50.)
sctpropsstruct.snomt0 = snomt0
print, '(K) snomt0 = ', snomt0
; Sigma in nomt0

t0string = '128270000.000'
sctpropsstruct.t0string = t0string
print, '(-) t0string = ', t0string
; zero for my time reference

endif ; if sct eq 'mer1'


if sct eq 'mer2' then begin
pdsfile = '2TRANSFORMED.TAB'
sctpropsstruct.pdsfile = pdsfile
print, '(-) pdsfile = ', pdsfile
; Location of raw data
	
nraw = 9176L
sctpropsstruct.nraw = nraw
print, '(-) nraw = ', nraw
; Number of lines in raw datafile

nsplit = 5315L
sctpropsstruct.nsplit = nsplit
print, '(-) nsplit = ', nsplit
; Backshell separates and IMU-B data ends

;tdetect = 2110.6250
;sctpropsstruct.tdetect = tdetect
;print, '(s) tdetect = ', tdetect
; Time of atm detection, determined manually

;tchute = 2336.625
;sctpropsstruct.tchute = tchute
;print, '(s) tchute = ', tchute
; Time of chute opening, determined manually

r0 = double(3392.3e3) ; MER-2 from MOLA/NSSDC
sctpropsstruct.r0 = r0
print, '(m) Planetary radius at impact site = ', r0

if uncert eq 1 then height_0 = r0 + 100e3
if uncert eq 0 then height_0 = r0 + 118e3
sctpropsstruct.height_0 = height_0
print, '(m) height_0 = ', height_0
; Top of region used for Hfit in top BC
; The atmospheric structure reconstruction is very sensitive to this

npadtop = 60L
sctpropsstruct.npadtop = npadtop
print, '(-) npadtop = ', npadtop
; Difference between top of uncert=0 and uncert=1 atmo recons

ndetect = 40L
if uncert eq 1 then ndetect = ndetect + npadtop
sctpropsstruct.ndetect = ndetect
print, '(-) ndetect = ', ndetect
; atm detection, determined manually

nchute = 1004L
sctpropsstruct.nchute = nchute
print, '(-) nchute = ', nchute
; chute opening, determined manually

nalpha = 190L 
sctpropsstruct.nalpha = nalpha
print, '(-) nalpha = ', nalpha
; Where do I start considering alpha?

i0 = 4677
sctpropsstruct.i0 = i0
print, '(-) i0 = ', i0
i1 = 4678
sctpropsstruct.i1 = i1
print, '(-) i1 = ', i1
; Couple of blank datapoints that need filling in by read_pds.pro

m = double(827.0)
sctpropsstruct.m = m
print, '(kg) Sct mass = ', m
; Mass of spacecraft, kg
; Desai AIAA-2004-5092, fig2

nomt0 = double(160.)
sctpropsstruct.nomt0 = nomt0
print, '(K) nomt0 = ', nomt0
; DSH fitted temp at height_0 using nominal recon, no uncert.

snomt0 = double(50.)
sctpropsstruct.snomt0 = snomt0
print, '(K) snomt0 = ', snomt0
; Sigma in nomt0

t0string = '126460000.000'
sctpropsstruct.t0string = t0string
print, '(-) t0string = ', t0string
; zero for my time reference
endif ; if sct eq 'mer2'



datafile = sct + '_pds.dat' 
sctpropsstruct.datafile = datafile
print, '(-) Datafile = ', datafile
; Name of file containing acc data

outdir = 'output/'
sctpropsstruct.outdir = outdir
print, '(-) outdir = ', outdir
; Directory where all (there are a lot) output files go

trustxy = double(0.)
sctpropsstruct.trustxy = trustxy
print, '(-) trustxy = ', trustxy
; 0 = use az only in get_acc
; 1 = use (ax^2 + ay^2 + az^2)^0.5 in get_acc

dt = double(0.25)
sctpropsstruct.dt = dt
print, '(s) dt = ', dt
; Timestep, s

n = 0L
sctpropsstruct.n = n
print, '(-) n will be set by read_pds.pro ', n
; Number of timesteps/measurements

nsection = double(1e6)
sctpropsstruct.nsection = nsection
print, '(-) nsection = ', nsection
; width of a subset of datapoints are given for interpolation in RK4
; see derivs.pro for usage

a = double(0.25*!pi * 2.648^2)
sctpropsstruct.a = a
print, '(m2) Sct area = ', a
; Area of spacecraft, m2
; use diameter from Schoenenbeger et al. AIAA-2005-0056

sa = double(0.01)
sctpropsstruct.sa = sa
print, '(m s-2) Sigma in acc = ', sa
; Arbitrarily decide that sigma = 0.01 ms-2, or 3x noise level

alimit = double(0.02)
sctpropsstruct.alimit = alimit 
print, '(m s-2) Limit on an before I try CN/CA = ', alimit
; Arbitrarily decide what this limit is...

range_0 = double(10e3)
sctpropsstruct.range_0 = range_0
print, '(m) range_0 = ', range_0
; Width of region used for Hfit in top BC
; The atmospheric structure reconstruction is very sensitive to this

convergence = double(1e-3)
sctpropsstruct.convergence = convergence
print, '(-) convergence criterion = ', convergence
; Test whether aerodynamics have converged yet

nmonte_traj = double(1000.0)
if uncert eq 0. then nmonte_traj = double(1.)
sctpropsstruct.nmonte_traj = nmonte_traj
print, '(-) nmonte_traj = ', nmonte_traj
; Number of Monte Carlo iterations

nmonte_atm = double(1000.0)
if uncert eq 0. then nmonte_atm = double(1.)
sctpropsstruct.nmonte_atm = nmonte_atm
print, '(-) nmonte_atm = ', nmonte_atm
; Number of Monte Carlo iterations

nmonte_hiz = double(3.0)
if uncert eq 0. then nmonte_hiz = double(1.)
sctpropsstruct.nmonte_hiz = nmonte_hiz
print, '(-) nmonte_hiz = ', nmonte_hiz
; Number of Monte Carlo iterations

blocksize = 10L
sctpropsstruct.blocksize = blocksize
print, '(-) blocksize = ', blocksize
; Number of points averaged in a hiz block

nblocks = 25L
sctpropsstruct.nblocks = nblocks
print, '(-) nblocks = ', nblocks
; Number of hiz blocks

zerofraclimit = double(0.8)
sctpropsstruct.zerofraclimit = zerofraclimit
print, '(-) zerofraclimit = ', zerofraclimit
; Min fraction of good points in a hiz block for average to be formed

badrunlimit = double(0.8)
sctpropsstruct.badrunlimit = badrunlimit
print, '(-) badrunlimit = ', badrunlimit
; Min fraction of good hiz recons at specific altitude necessary
; for ensemble statistics to be calculated

itlimit = double(100)
sctpropsstruct.itlimit = itlimit
print, '(-) itlimit = ', itlimit
; Give up on atm recon if it iterates more than this value

entrystate = {t:double(0.0), r:double(0.0), lat:double(0.0), lon:double(0.0), speed:double(0.0), gamma:double(0.0), psi:double(0.0), st:double(0.0), sr:double(0.0), slat:double(0.0), slon:double(0.0), sspeed:double(0.0), sgamma:double(0.0), spsi:double(0.0), rot_factor:double(0.0)}

; Entry states based on stated sources

if sct eq 'mer1' then begin
entrystate.t = double( 8194.6250 ) 
; Desai chute deploy (mortar) time in AIAA-2004-5092, Table 4, last column
; 250.25s prior to mortar
entrystate.r     = double( 3522.2 * 1e3 ) ; PDS MER_MISSION.CAT file
entrystate.lat   = double(  -2.9 ) ; fit
entrystate.lon   = double( 340.9 ) ; fit
entrystate.speed = double( 5.70 * 1e3 ) ; Desai AIAA-2004-5092, p2
entrystate.gamma = double( -11.5 * (-1.)) ; Desai AIAA-2004-5092, p2
entrystate.psi   = double( 86.5 ) 
; fit, guided by Golombek Site Selection MER JGR special issue
entrystate.rot_factor = double(0.)
; fit to correct r0 (above), lat/lon (Desai AIAA-2004-5092, p1), 
; and reasonable vrel at bridle cut
endif ; if sct eq 'mer1'

if sct eq 'mer2' then begin
entrystate.t     = double( 2085.6250 ) 
; Desai chute deploy (mortar) time in AIAA-2004-5092, Table 4, last column
; 251.00s prior to mortar
entrystate.r     = double( 3522.2 * 1e3 ) ; PDS MER_MISSION.CAT file
entrystate.lat   = double(  -17.7 ) ; fit
entrystate.lon   = double( 161.8 ) ; fit
entrystate.speed = double( 5.63 * 1e3 ) ; Desai AIAA-2004-5092, p2
entrystate.gamma = double( -11.5 * (-1.)) ; Desai AIAA-2004-5092, p2
entrystate.psi   = double( 79.0 )  
; fit, guided by Golombek Site Selection MER JGR special issue
entrystate.rot_factor = double(0.)
; fit to correct r0 (above), lat/lon (Desai AIAA-2004-5092, p1), 
; and reasonable vrel at bridle cut
endif ; if sct eq 'mer2'

entrystate.st = double(0.)
entrystate.sr = double(1.7e3)
entrystate.slat = double(0.04)
entrystate.slon = double(0.01)
entrystate.sspeed = double(0.7)
entrystate.sgamma = double(0.02)
entrystate.spsi = double(0.02)
; Uncertainties, arbitrarily assumed to be the same as in the MPF PDS case

sctpropsstruct = create_struct(sctpropsstruct, 'entrystate', entrystate)




; MAKE ALL DP AT SOME POINT

; ***** FMF *****
; eyeballed from Fig 6
alpha_fmf = [0.,6.,10.,16.,20.]
ca_fmf = dblarr(5)+2.
cn_fmf = dblarr(5)
cm_fmf = dblarr(5)
na_fmf = n_elements(alpha_fmf)

; ***** TRANS REGION *****
; eyeballed from Fig 6
alpha_trans = [0.,6.,10.,16.,20.]
na_trans = n_elements(alpha_trans)
kn_trans = [1e-1,3e-1,1e0,3e0,1e1,3e1,1e2]
nkn_trans = n_elements(kn_trans)

ca_transkn1e2 = dblarr(5)+2.
ca_transkn3e1 = dblarr(5)+2.
ca_transkn1e1 = dblarr(5)+2.
ca_transkn3e0 = dblarr(5)+2.
ca_transkn1e0 = dblarr(5)+2.
ca_transkn3em1 = dblarr(5)+2.
ca_transkn1em1 = dblarr(5)+2.
ca_transalphakn = dblarr(na_trans, nkn_trans)
ca_transalphakn(*,0) = ca_transkn1em1(*)
ca_transalphakn(*,1) = ca_transkn3em1(*)
ca_transalphakn(*,2) = ca_transkn1e0(*)
ca_transalphakn(*,3) = ca_transkn3e0(*)
ca_transalphakn(*,4) = ca_transkn1e1(*)
ca_transalphakn(*,5) = ca_transkn3e1(*)
ca_transalphakn(*,6) = ca_transkn1e2(*)

cn_transkn1e2 = dblarr(5)
cn_transkn3e1 = dblarr(5)
cn_transkn1e1 = dblarr(5)
cn_transkn3e0 = dblarr(5)
cn_transkn1e0 = dblarr(5)
cn_transkn3em1 = dblarr(5)
cn_transkn1em1 = dblarr(5)
cn_transalphakn = dblarr(na_trans, nkn_trans)
cn_transalphakn(*,0) = cn_transkn1em1(*)
cn_transalphakn(*,1) = cn_transkn3em1(*)
cn_transalphakn(*,2) = cn_transkn1e0(*)
cn_transalphakn(*,3) = cn_transkn3e0(*)
cn_transalphakn(*,4) = cn_transkn1e1(*)
cn_transalphakn(*,5) = cn_transkn3e1(*)
cn_transalphakn(*,6) = cn_transkn1e2(*)

cm_transkn1e2 = dblarr(5)
cm_transkn3e1 = dblarr(5)
cm_transkn1e1 = dblarr(5)
cm_transkn3e0 = dblarr(5)
cm_transkn1e0 = dblarr(5)
cm_transkn3em1 = dblarr(5)
cm_transkn1em1 = dblarr(5)
cm_transalphakn = dblarr(na_trans, nkn_trans)
cm_transalphakn(*,0) = cm_transkn1em1(*)
cm_transalphakn(*,1) = cm_transkn3em1(*)
cm_transalphakn(*,2) = cm_transkn1e0(*)
cm_transalphakn(*,3) = cm_transkn3e0(*)
cm_transalphakn(*,4) = cm_transkn1e1(*)
cm_transalphakn(*,5) = cm_transkn3e1(*)
cm_transalphakn(*,6) = cm_transkn1e2(*)

; ***** CONT-V REGION *****
; From email, later change to eyeballed data coz it's public

alpha_contv = [0.,2.,4.,6.,11.,16.]
na_contv = n_elements(alpha_contv)
v_contv = reverse( [5534., 5400., 5200., 5000., 4800., 4600., 4400., 4200., 4000., $
 3800., 3600., 3400., 3200., 3000., 2800., 2600., 2400., 2200., 2000.] )
nv_contv = n_elements(v_contv)
ma_contv = reverse( [26.650, 25.437, 24.155, 23.033, 21.963, 20.951, 19.954, $
 18.987, 18.032, $
 17.073, 16.138, 15.212, 14.289, 13.366, 12.455, 11.543, 10.635, 9.732, 8.831] )
nma_contv = n_elements(ma_contv)
; want v increasing from minimum...

ca_contv5534 = dblarr(6)+2.
ca_contv5400 = dblarr(6)+2.
ca_contv5200 = dblarr(6)+2.
ca_contv5000 = dblarr(6)+2.
; fill in 4800
ca_contv4600 = dblarr(6)+2.
; fill in 4400, 4200
ca_contv4000 = dblarr(6)+2.
; fill in 3800
ca_contv3600 = dblarr(6)+2.
; fill in 34, 32, 30, 28, 26
ca_contv2400 = dblarr(6)+2.
; fill in 22
ca_contv2000 = dblarr(6)+2.

ca_contv4800 = 0.5 * ( ca_contv5000  + ca_contv4600 )
ca_contv4800(1) =  2.
ca_contv4400 = (2./3.) * ca_contv4600  + (1./3.) * ca_contv4000
ca_contv4400(1) =  2.
ca_contv4200 = (1./3.) * ca_contv4600  + (2./3.) * ca_contv4000
ca_contv4200(1) =  2.
ca_contv3800 = 0.5 * (ca_contv4000 +  ca_contv3600 )
ca_contv3800(1) =  2.
ca_contv3800(2) =  2.
ca_contv3800(3) =  2.
ca_contv3400 = (1./6.) * ca_contv2400 + (5./6.) * ca_contv3600
ca_contv3400(1) =  2.
ca_contv3200 = (2./6.) * ca_contv2400 + (4./6.) * ca_contv3600
ca_contv3200(1) =  2.
ca_contv3200(2) =  2.
ca_contv3200(3) =  2.
ca_contv3000 = (3./6.) * ca_contv2400 + (3./6.) * ca_contv3600
ca_contv3000(1) =  2.
ca_contv2800 = (4./6.) * ca_contv2400 + (2./6.) * ca_contv3600
ca_contv2800(1) =  2.
ca_contv2800(2) =  2.
ca_contv2600 = (5./6.) * ca_contv2400 + (1./6.) * ca_contv3600
ca_contv2600(1) =  2.
ca_contv2200 = 0.5 * ( ca_contv2400  + ca_contv2000 )
ca_contv2200(1) =  2.

ca_contvalphav = dblarr(na_contv, nv_contv)
ca_contvalphav(*,0) = ca_contv2000(*)
ca_contvalphav(*,1) = ca_contv2200(*)
ca_contvalphav(*,2) = ca_contv2400(*)
ca_contvalphav(*,3) = ca_contv2600(*)
ca_contvalphav(*,4) = ca_contv2800(*)
ca_contvalphav(*,5) = ca_contv3000(*)
ca_contvalphav(*,6) = ca_contv3200(*)
ca_contvalphav(*,7) = ca_contv3400(*)
ca_contvalphav(*,8) = ca_contv3600(*)
ca_contvalphav(*,9) = ca_contv3800(*)
ca_contvalphav(*,10) = ca_contv4000(*)
ca_contvalphav(*,11) = ca_contv4200(*)
ca_contvalphav(*,12) = ca_contv4400(*)
ca_contvalphav(*,13) = ca_contv4600(*)
ca_contvalphav(*,14) = ca_contv4800(*)
ca_contvalphav(*,15) = ca_contv5000(*)
ca_contvalphav(*,16) = ca_contv5200(*)
ca_contvalphav(*,17) = ca_contv5400(*)
ca_contvalphav(*,18) = ca_contv5534(*)


cn_contv5534 = dblarr(6)
cn_contv5400 = dblarr(6)
cn_contv5200 = dblarr(6)
cn_contv5000 = dblarr(6)
; fill in 4800
cn_contv4600 = dblarr(6)
; fill in 4400, 4200
cn_contv4000 = dblarr(6)
; fill in 3800
cn_contv3600 = dblarr(6)
; fill in 34, 32, 30, 28, 26
cn_contv2400 = dblarr(6)
; fill in 22
cn_contv2000 = dblarr(6)

cn_contv4800 = 0.5 * ( cn_contv5000  + cn_contv4600 )
cn_contv4800(1) =  0.
cn_contv4400 = (2./3.) * cn_contv4600  + (1./3.) * cn_contv4000
cn_contv4400(1) =  0.
cn_contv4200 = (1./3.) * cn_contv4600  + (2./3.) * cn_contv4000
cn_contv4200(1) =  0.
cn_contv3800 = 0.5 * (cn_contv4000 +  cn_contv3600 )
cn_contv3800(1) =  0.
cn_contv3800(2) =  0.
cn_contv3800(3) =  0.
cn_contv3400 = (1./6.) * cn_contv2400 + (5./6.) * cn_contv3600
cn_contv3400(1) =  0.
cn_contv3200 = (2./6.) * cn_contv2400 + (4./6.) * cn_contv3600
cn_contv3200(1) =  0.
cn_contv3200(2) =  0.
cn_contv3200(3) =  0.
cn_contv3000 = (3./6.) * cn_contv2400 + (3./6.) * cn_contv3600
cn_contv3000(1) =  0.
cn_contv2800 = (4./6.) * cn_contv2400 + (2./6.) * cn_contv3600
cn_contv2800(1) =  0.
cn_contv2800(2) =  0.
cn_contv2600 = (5./6.) * cn_contv2400 + (1./6.) * cn_contv3600
cn_contv2600(1) =  0.
cn_contv2200 = 0.5 * ( cn_contv2400  + cn_contv2000 )
cn_contv2200(1) =  0.

cn_contvalphav = dblarr(na_contv, nv_contv)
cn_contvalphav(*,0) = cn_contv2000(*)
cn_contvalphav(*,1) = cn_contv2200(*)
cn_contvalphav(*,2) = cn_contv2400(*)
cn_contvalphav(*,3) = cn_contv2600(*)
cn_contvalphav(*,4) = cn_contv2800(*)
cn_contvalphav(*,5) = cn_contv3000(*)
cn_contvalphav(*,6) = cn_contv3200(*)
cn_contvalphav(*,7) = cn_contv3400(*)
cn_contvalphav(*,8) = cn_contv3600(*)
cn_contvalphav(*,9) = cn_contv3800(*)
cn_contvalphav(*,10) = cn_contv4000(*)
cn_contvalphav(*,11) = cn_contv4200(*)
cn_contvalphav(*,12) = cn_contv4400(*)
cn_contvalphav(*,13) = cn_contv4600(*)
cn_contvalphav(*,14) = cn_contv4800(*)
cn_contvalphav(*,15) = cn_contv5000(*)
cn_contvalphav(*,16) = cn_contv5200(*)
cn_contvalphav(*,17) = cn_contv5400(*)
cn_contvalphav(*,18) = cn_contv5534(*)


cm_contv5534 = dblarr(6)
cm_contv5400 = dblarr(6)
cm_contv5200 = dblarr(6)
cm_contv5000 = dblarr(6)
; fill in 4800
cm_contv4600 = dblarr(6)
; fill in 4400, 4200
cm_contv4000 = dblarr(6)
; fill in 3800
cm_contv3600 = dblarr(6)
; fill in 34, 32, 30, 28, 26
cm_contv2400 = dblarr(6)
; fill in 22
cm_contv2000 = dblarr(6)

cm_contv4800 = 0.5 * ( cm_contv5000  + cm_contv4600 )
cm_contv4800(1) =  0.
cm_contv4400 = (2./3.) * cm_contv4600  + (1./3.) * cm_contv4000
cm_contv4400(1) =  0.
cm_contv4200 = (1./3.) * cm_contv4600  + (2./3.) * cm_contv4000
cm_contv4200(1) =  0.
cm_contv3800 = 0.5 * (cm_contv4000 +  cm_contv3600 )
cm_contv3800(1) =  0.
cm_contv3800(2) =  0.
cm_contv3800(3) =  0.
cm_contv3400 = (1./6.) * cm_contv2400 + (5./6.) * cm_contv3600
cm_contv3400(1) =  0.
cm_contv3200 = (2./6.) * cm_contv2400 + (4./6.) * cm_contv3600
cm_contv3200(1) =  0.
cm_contv3200(2) =  0.
cm_contv3200(3) =  0.
cm_contv3000 = (3./6.) * cm_contv2400 + (3./6.) * cm_contv3600
cm_contv3000(1) =  0.
cm_contv2800 = (4./6.) * cm_contv2400 + (2./6.) * cm_contv3600
cm_contv2800(1) =  0.
cm_contv2800(2) =  0.
cm_contv2600 = (5./6.) * cm_contv2400 + (1./6.) * cm_contv3600
cm_contv2600(1) =  0.
cm_contv2200 = 0.5 * ( cm_contv2400  + cm_contv2000 )
cm_contv2200(1) =  0.

cm_contvalphav = dblarr(na_contv, nv_contv)
cm_contvalphav(*,0) = cm_contv2000(*)
cm_contvalphav(*,1) = cm_contv2200(*)
cm_contvalphav(*,2) = cm_contv2400(*)
cm_contvalphav(*,3) = cm_contv2600(*)
cm_contvalphav(*,4) = cm_contv2800(*)
cm_contvalphav(*,5) = cm_contv3000(*)
cm_contvalphav(*,6) = cm_contv3200(*)
cm_contvalphav(*,7) = cm_contv3400(*)
cm_contvalphav(*,8) = cm_contv3600(*)
cm_contvalphav(*,9) = cm_contv3800(*)
cm_contvalphav(*,10) = cm_contv4000(*)
cm_contvalphav(*,11) = cm_contv4200(*)
cm_contvalphav(*,12) = cm_contv4400(*)
cm_contvalphav(*,13) = cm_contv4600(*)
cm_contvalphav(*,14) = cm_contv4800(*)
cm_contvalphav(*,15) = cm_contv5000(*)
cm_contvalphav(*,16) = cm_contv5200(*)
cm_contvalphav(*,17) = cm_contv5400(*)
cm_contvalphav(*,18) = cm_contv5534(*)



; ***** CONT-MA REGION *****
; From email, later change to eyeballed data coz it's public

alpha_contma = [0.,6.,11.,16.]
na_contma = n_elements(alpha_contma)
v_contma = reverse( [1316.83, 1089.41, 891.893, 679.062, 471.700, 359.936] )
nv_contma = n_elements(v_contma)
ma_contma = reverse( [6.275, 5.122, 4.123, 3.048, 2.038, 1.512] )
nma_contma = n_elements(ma_contma)

cafb_contma6 = dblarr(4)+2.
cafb_contma5 = dblarr(4)+2.
cafb_contma4 = dblarr(4)+2.
cafb_contma3 = dblarr(4)+2.
cafb_contma2 = dblarr(4)+2.
cafb_contma1 = dblarr(4)+2.
cafb_contmaalphama = dblarr(na_contma, nma_contma)
cafb_contmaalphama(*,0) = cafb_contma1(*)
cafb_contmaalphama(*,1) = cafb_contma2(*)
cafb_contmaalphama(*,2) = cafb_contma3(*)
cafb_contmaalphama(*,3) = cafb_contma4(*)
cafb_contmaalphama(*,4) = cafb_contma5(*)
cafb_contmaalphama(*,5) = cafb_contma6(*)

; correct for base pressure
a0 = 0.
a1 = 0.
a2 = 0.
a3 = 0.
ca_contmaalphama = dblarr(na_contma, nma_contma)
i=0
while i lt nma_contma do begin
ca_contmaalphama(*,i) = cafb_contmaalphama(*,i) + $
 a0 + a1/ma_contma(i) + a2/ma_contma(i)^2 + a3/ma_contma(i)^3
i=i+1
endwhile


cn_contma6 = dblarr(4)
cn_contma5 = dblarr(4)
cn_contma4 = dblarr(4)
cn_contma3 = dblarr(4)
cn_contma2 = dblarr(4)
cn_contma1 = dblarr(4)
cn_contmaalphama = dblarr(na_contma, nma_contma)
cn_contmaalphama(*,0) = cn_contma1(*)
cn_contmaalphama(*,1) = cn_contma2(*)
cn_contmaalphama(*,2) = cn_contma3(*)
cn_contmaalphama(*,3) = cn_contma4(*)
cn_contmaalphama(*,4) = cn_contma5(*)
cn_contmaalphama(*,5) = cn_contma6(*)

cm_contma6 = dblarr(4)
cm_contma5 = dblarr(4)
cm_contma4 = dblarr(4)
cm_contma3 = dblarr(4)
cm_contma2 = dblarr(4)
cm_contma1 = dblarr(4)
cm_contmaalphama = dblarr(na_contma, nma_contma)
cm_contmaalphama(*,0) = cm_contma1(*)
cm_contmaalphama(*,1) = cm_contma2(*)
cm_contmaalphama(*,2) = cm_contma3(*)
cm_contmaalphama(*,3) = cm_contma4(*)
cm_contmaalphama(*,4) = cm_contma5(*)
cm_contmaalphama(*,5) = cm_contma6(*)


; ***** MERGE SUPER/HYPER SONIC PARTS OF CONTINUUM *****

alpha_cont = dindgen(17)
na_cont = n_elements(alpha_cont)
v_cont = [v_contma, v_contv]
nv_cont = n_elements(v_cont)
ma_cont = [ma_contma, ma_contv]
nma_cont = n_elements(ma_cont)

ca_cont = dblarr(na_cont, nv_cont)
cn_cont = dblarr(na_cont, nv_cont)
cm_cont = dblarr(na_cont, nv_cont)

i=0
while i lt nv_cont do begin

if i lt nma_contma then begin
ca_cont(*,i) = interpol( ca_contmaalphama(*,i), alpha_contma, alpha_cont)
cn_cont(*,i) = interpol( cn_contmaalphama(*,i), alpha_contma, alpha_cont)
cm_cont(*,i) = interpol( cm_contmaalphama(*,i), alpha_contma, alpha_cont)
endif
if i ge nma_contma then begin
j=i-nma_contma
ca_cont(*,i) = interpol( ca_contvalphav(*,j), alpha_contv, alpha_cont)
cn_cont(*,i) = interpol( cn_contvalphav(*,j), alpha_contv, alpha_cont)
cm_cont(*,i) = interpol( cm_contvalphav(*,j), alpha_contv, alpha_cont)
endif

i=i+1
endwhile


; ***** END MAKING ARRAYS *****

; kn gt 1e3 -> FMF
; kn gt 1.06e-3 and kn lt 1e3 -> Trans, linear in logKn
; kn lt 1.06e-3 -> Cont

; Ma gt 12 -> C(v)
; Ma gt 8 and Ma lt 12 -> Mix of C(v) and C(Ma)
; Ma lt 8 -> C(Ma)

; Ma gt 8.831 -> CA fb only
; Ma lt 8.831 and Ma gt 6.275 -> Max of CAfb and CAtotal
; Ma lt 6.275 -> CA total

; Need Kn, Ma, v, an/aa
; Return CA, CN, Cm, alpha

; kn ge knfmf -> FMF
knfmf = 1e3 
; kn ge kntoptrans and kn lt knfmf -> FMF/TRANS
kntoptrans = max(kn_trans) 
; kn ge knbottrans and kn lt kntoptrans -> TRANS
knbottrans = min(kn_trans) 
; kn ge kncont and kn lt knbottrans -> CONT/TRANS
kncont = 1.06e-3 
; kn lt kncont -> CONT

; ma ge max(ma_cont) -> assume ma=max(ma_cont)
; use max(ma) not max(v) because it gives same result but has no
; possibility of v fitting one criterion, ma another
matopweight=12. ; ma ge matopweight -> interpolate in v only
; ma ge mabotweight and ma lt matopweight -> interpolate in both v and Ma
mabotweight=8.0 
; ma lt matopweight -> interpolate in Ma only
; ma lt min(ma_cont) -> assume ma=max(ma_cont)

; Note that FB correction is automatically weighted because of gap between 
; Ma = 6.3 and 8.8 datapoints

sca = 0.05 ; Relative error in CA
alphaerr = 99. ; Alpha is beyond max range...

; aerodb = {alpha_fmf:alpha_fmf, na_fmf:na_fmf, ca_fmf:ca_fmf, cn_fmf:cn_fmf, cm_fmf:cm_fmf, alpha_trans:alpha_trans, na_trans:na_trans, kn_trans:kn_trans, nkn_trans:nkn_trans, ca_transalphakn:ca_transalphakn, cn_transalphakn:cn_transalphakn, cm_transalphakn:cm_transalphakn, alpha_cont:alpha_cont, na_cont:na_cont, v_cont:v_cont, nv_cont:nv_cont, ma_cont:ma_cont, nma_cont:nma_cont, ca_cont:ca_cont, cn_cont:cn_cont, cm_cont:cm_cont, knfmf:knfmf, kntoptrans:kntoptrans, knbottrans:knbottrans, kncont:kncont, matopweight:matopweight, mabotweight:mabotweight, sca:sca, alphaerr:alphaerr} 

aerodb = {alpha_fmf:alpha_fmf, na_fmf:na_fmf, ca_fmf:ca_fmf, cn_fmf:cn_fmf, cm_fmf:cm_fmf, alpha_trans:alpha_trans, na_trans:na_trans, kn_trans:kn_trans, nkn_trans:nkn_trans, ca_transalphakn:ca_transalphakn, cn_transalphakn:cn_transalphakn, cm_transalphakn:cm_transalphakn, alpha_cont:alpha_cont, na_cont:na_cont, v_cont:v_cont, nv_cont:nv_cont, ma_cont:ma_cont, nma_cont:nma_cont, ca_cont:ca_cont, cn_cont:cn_cont, cm_cont:cm_cont, alpha_contma:alpha_contma, na_contma:na_contma, v_contma:v_contma, nv_contma:nv_contma, ma_contma:ma_contma, nma_contma:nma_contma, ca_contmaalphama:ca_contmaalphama, cn_contmaalphama:cn_contmaalphama, cm_contmaalphama:cm_contmaalphama, alpha_contv:alpha_contv, na_contv:na_contv, v_contv:v_contv, nv_contv:nv_contv, ma_contv:ma_contv, nma_contv:nma_contv, ca_contvalphav:ca_contvalphav, cn_contvalphav:cn_contvalphav, cm_contvalphav:cm_contvalphav, knfmf:knfmf, kntoptrans:kntoptrans, knbottrans:knbottrans, kncont:kncont, matopweight:matopweight, mabotweight:mabotweight, sca:sca, alphaerr:alphaerr} 

aerosigmatracker = aerodb

sctpropsstruct = create_struct(sctpropsstruct, 'aerodb', aerodb)

help, sctpropsstruct, /struct

end
